/****************************************************************************
**
** This file is part of PLCEdit, an open-source cross-platform editor 
** for PLC source files (e.g. from Moeller, CoDeSys and Siemens).
** Copyright (C) 2005-2009  M. Rehfeldt
**
** This software uses classes of Trolltech Qt toolkit and is freeware. 
** This file may be used under the terms of the GNU General Public License 
** version 2.0 or (at your option) any later version as published by the 
** Free Software Foundation and appearing in the file LICENSE.GPL included 
** in the packaging of this file. 
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** Contact e-mail: M. Rehfeldt <info@plcedit.org>
** Program URL   : http://www.plcedit.org
**
****************************************************************************/


#include <QtGui>

#include "pagectrl.h"
#include "pagedata.h"
#include "highlighter_dec.h"
#include "highlighter_ins.h"
#include "editor.h"


PageCtrl::PageCtrl(QWidget *parent)
    : QSplitter(parent)
{ //! creating function
	
  //widget->visual settings for parent
  this->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  this->setMidLineWidth(0);
  this->setChildrenCollapsible(false); 
  this->show();
  
  //create FileList as child of PageCtrl
  FileList = new QListWidget(this);
  FileList->setFrameStyle(QFrame::Box | QFrame::Raised);
  FileList->setMidLineWidth(0);
  FileList->setAlternatingRowColors(true);
  FileList->setMinimumWidth(100);
  FileList->setMaximumWidth(165);
  FileList->move(0,0);
  FileList->hide();
  
  //create TabCtrl as child of PageCtrl
  TabCtrl  = new QTabWidget(this);      
  TabCtrl->setUsesScrollButtons(true);
  TabCtrl->setLayoutDirection(Qt::LeftToRight);
  TabCtrl->hide();
  
  //create delete page button
  delPageButton = new QToolButton(this);
  TabCtrl->setCornerWidget(delPageButton, Qt::TopRightCorner);
  delPageButton->setAutoRaise(true);
  #ifdef Q_WS_MAC
    delPageButton->setIcon(QIcon(":/images/delPageButtonMac_16x16.png"));
  #else
    delPageButton->setIcon(QIcon(":/images/delPageButton_16x16.png"));
  #endif

  flg_inhibit_createHighlighters = false; 	//presetting inhibit highlighters flag, needed for start up

  
  //diverse connections
  connect(FileList, SIGNAL(currentRowChanged(int)), TabCtrl,SLOT(setCurrentIndex(int)));			//change Tab with FileList change
  connect(TabCtrl , SIGNAL(currentChanged(int)), 	this, 	SLOT(setFileListRow(int)));				//change FileList with Tab change
  connect(TabCtrl, 	SIGNAL(currentChanged(int)), 	this, 	SIGNAL(currentChanged(int)));			//send SIGNAL with Tab change
  connect(TabCtrl, 	SIGNAL(currentChanged(int)), 	this, 	SLOT(createHighlighters(int)));			//create syntax highlighters with Tab change
  connect(this,  	SIGNAL(pageDeleted(int)), 		this, 	SLOT(createHighlighters(int)));			//create syntax highlighters if prev. page was deleted
  connect(TabCtrl, 	SIGNAL(currentChanged(int)), 	this, 	SLOT(deleteCurrentHighlighters(int)));	//delete syntax highlighters with Tab change
  connect(TabCtrl, 	SIGNAL(currentChanged(int)), 	this, 	SLOT(configureCurrentEditor(int)));		//set lines highlighters with Tab change
  connect(delPageButton, SIGNAL(clicked()), 		this, 	SLOT(deletePage()));					//get access to delete pages function
}


PageCtrl::~PageCtrl()
{ //virtual destructor

  delete FileList;
  delete TabCtrl;
}




//=============================================================================
// - - - public SLOTs - - -
void PageCtrl::changeSplitMode()
{ //! reversing splitter orientation
  
  if (TabCtrl->count() < 1)//if1
    return; 
  
  //if counter > 0 set splitter orientation
  QSplitter *splitter1 = qobject_cast<QSplitter *>(TabCtrl->currentWidget());

  if (splitMode == "v")
    splitter1->setOrientation(Qt::Vertical);

  if (splitMode == "h")
    splitter1->setOrientation(Qt::Horizontal);
}


void PageCtrl::changeInputWindow()
{ //! toggle focus and cursor for declaratins and Instructions

  if ((TabCtrl->count() < 1) || (currentFileType() == "GLOBAL_VARIABLE_LIST"))
    return; 
  if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->hasFocus())
  {	TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->setFocus(); return; }
  
  if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->hasFocus())
  {	TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->setFocus(); return; }
  	
  //if nothing focused, set focus to Declarations
  TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->setFocus();	
}


void PageCtrl::configureCurrentEditor(int i)
{ //! set lines highlighter
  //! \param i is only needed for signal-slot handling

  if (TabCtrl->count() < 1)
    return; //return if no page is active
    
  TabCtrl->currentWidget()->findChild<Editor*>("TE1")->enableHighlightLines = flg_enableHighlightLines;
  TabCtrl->currentWidget()->findChild<Editor*>("TE1")->hiLineColor = hiLineColor;
  TabCtrl->currentWidget()->findChild<Editor*>("TE1")->printArea = printArea;
  if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
  {
    TabCtrl->currentWidget()->findChild<Editor*>("TE2")->enableHighlightLines = flg_enableHighlightLines;
    TabCtrl->currentWidget()->findChild<Editor*>("TE2")->hiLineColor = hiLineColor;
    TabCtrl->currentWidget()->findChild<Editor*>("TE2")->printArea = printArea;
  }
}


void PageCtrl::copy()
{ //! copy from current inputwindow

  if (TabCtrl->count() < 1)
    return; 
  if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->hasFocus())
  	TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->copy();
  if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
    if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->hasFocus())
      TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->copy();
}


void PageCtrl::createHighlighters(int i)
{ //! slot create syntax highlighters of current page
  //! \param i is only needed for signal-slot handling

  if ((flg_inhibit_createHighlighters == true) || (flg_activate_highlighters == false) || (TabCtrl->count() < 1))
  	return; //return if the open function is still in progress or highlighters not allowed or no page exists
  if ((TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_highlighters) == "ON")
    return; //return if the highlighters were already created

  //set wait cursor
  QApplication::setOverrideCursor(Qt::WaitCursor);
  
  //create highlighters for declarations and if exists also the instructions TextEdit components
  highlighter_dec     = new Highlighter_Dec(qobject_cast<QTextEdit*>(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1"))->document());
  highlighter_dec->setObjectName("highlighter_dec");
  if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2") != 0)
  {
    highlighter_ins     = new Highlighter_Ins(qobject_cast<QTextEdit*>(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2"))->document());
    highlighter_ins->setObjectName("highlighter_ins");
  }
  
  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_highlighters = "ON"; //set label text to "ON"

  //restore cursor
  QApplication::restoreOverrideCursor();
}


void PageCtrl::cut()
{ //! cut from current inputwindow

  if (TabCtrl->count() < 1)
    return; 
  if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->hasFocus())
  	TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->cut();
  if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
    if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->hasFocus())
      TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->cut();
}


void PageCtrl::deleteCurrentHighlighters(int i)
{ //! slot delete syntax highlighters of current page
  //! \param i is only needed for signal-slot handling

  if ((flg_inhibit_createHighlighters == true) || (flg_activate_highlighters == true) || (TabCtrl->count() < 1))
  	return; //return if the open function is still in progress or highlighters allowed or no page exists
  if ((TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_highlighters) == "OFF")
    return; //return if the highlighters were already deleted
  
  //set wait cursor
  QApplication::setOverrideCursor(Qt::WaitCursor);
  //delete declarations highlighter and if exists also the instruction highlighter
  delete qobject_cast<Highlighter_Dec*>(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->document()->findChild<Highlighter_Dec*>("highlighter_dec"));
  if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2") != 0)
    delete qobject_cast<Highlighter_Ins*>(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->document()->findChild<Highlighter_Ins*>("highlighter_ins"));
  
  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_highlighters = "OFF";  //set label text to "OFF"

  //restore cursor
  QApplication::restoreOverrideCursor();
}


void PageCtrl::deletePage()
{ //! delete current page

  int index=0;
  
  //delete tab + list item if counter > 0
  if (TabCtrl->count() >= 0)
  {
    index = TabCtrl->currentIndex(); //get current index
    
    FileList->blockSignals(true); 
    TabCtrl->blockSignals(true); 

    fillRecentFilesList(index);
    FileList->takeItem(index);
    delete TabCtrl->currentWidget();
   
    FileList->blockSignals(false); 
    TabCtrl->blockSignals(false); 
   
    emit pageDeleted(index); //emit page deleted signal
  }
  
  //hide TabCtrl and FileList if counter < 1
  if (TabCtrl->count() < 1)
  {
  	FileList->hide(); TabCtrl->hide();
  }
  	
  emit modified(); //emit modiefied signal
}


void PageCtrl::newPage()
{ //! create a new page
  /*! pages contain a splitter, two inputwindows and some labels for page specific data
   *  PLCEdit uses no database to store file specific data. The file specific data are
   *  stored into hidden labels on the pages. Only the Splitter and the TextEdits are visible. 
   *  The content/components of the page can be edited by diverse public functions. 
   */
  /*!
   * page components: 
   * - splitter				= seperates the QTextEdits, parent is TabCtrl 
   *   - TE1 				= contains declarations, parent is splitter 
   *   - TE2 				= contains instructions, parent is splitter 
   *   - pageData			= contains the following data strings, parent is splitter 
   *     - pd_FilePath		= contains path to file without file name, parent is pageData 
   *     - pd_FileExt 		= contains file extension, parent is pageData 
   *     - pd_FileType		= contains POU type mark, parent is pageData 
   *     - pd_FncType		= contains type of function, parent is pageData 
   *     - pd_FileEnd 		= contains POU end mark, parent is pageData 
   *     - pd_PROGLNG  		= contains programming language of file, parent is pageData 
   *     - pd_KMHeader		= contains Moeller Sucosoft header, parent is pageData 
   *     - pd_KWDescription	= contains KW-Software description section, parent is pageData 
   *     - pd_KWProperties 	= contains KW-Software properties section, parent is pageData
   *     - pd_KWWorkSheetName = contains KW-Software work sheet name, parent is pageData 
   *     - pd_SiemensDBData = contains Siemens DB Data, parent is pageData
   *     - pd_highlighters	= contains highlighter is allowance, parent is pageData 
   *     .
   *   .
   * . 
   */
    
  //Splitter container -> items are countet below
  QSplitter *splitter = new QSplitter(TabCtrl);  	//create new QSplitter as child of TabCtrl
  splitter->setObjectName("splitter");             	//set ObjectName to splitter
  splitter->hide();									//hide splitter for faster file opening
  //TextEdit elements (0), (1)
  Editor *TE1      = new Editor(splitter);  		//create QTextEdit 1 as child of splitter
  Editor *TE2      = new Editor(splitter);  		//create QTextEdit 2 as child of splitter
  TE1->setObjectName("TE1");                      	//set ObjectName to TE1
  TE2->setObjectName("TE2");                      	//set ObjectName to TE2
  TE1->setFont(font);              					//set Font of TE1
  TE2->setFont(font);              					//set Font of TE2
  TE1->setTabStopWidth(tabStopWidth);              	//set TabStop width of TE1
  TE2->setTabStopWidth(tabStopWidth);              	//set TabStop width of TE2
  //page data
  PageData *pageData = new PageData(splitter);		//create the page data class as child of splitter
  pageData->setObjectName("pageData");				//set ObjectName to pageData
  pageData->pd_PROGLNG = prgLngStr;					//set pd_PROGLNG string
  
  
  //add List item with name 'untitled'
  FileList->addItem(tr("untitled"));
  //add splitter as new Tab to TabCtrl with name 'untitled'
  TabCtrl->addTab(splitter, tr("untitled"));
  //set new splitter to current widget
  TabCtrl->setCurrentWidget(splitter);
  //configure editor (lines highlighters and print area indicator for TE1 & TE2, needed to prevent uncontrolled indications)
  configureCurrentEditor(0);
  //set splitter orientation
  setCurrentSplitMode(splitMode);
  //set set split size
  setCurrentSplitSize(splitSize);
  //set tab orientation
  setTabPosition(tabPosition);
  //set List and Tab component visibility
  setFileListVisible(flg_fileListVisible);
  setTabCtrlVisible(true);

  //diverse connections
  connect(TE1, SIGNAL(cursorPositionChanged()), this, SLOT(cursorPositionChanged()));
  connect(TE2, SIGNAL(cursorPositionChanged()), this, SLOT(cursorPositionChanged()));
  
  //emit modified information
  emit modified();
}


void PageCtrl::paste()
{ //! paste into current inputwindow

  if (TabCtrl->count() < 1)
    return; 
  if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->hasFocus())
  	TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->paste();
  if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
    if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->hasFocus())
  	  TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->paste();
}


void PageCtrl::redo()
{ //! redo last input on current inputwindow

  if (TabCtrl->count() < 1)
    return; 
  if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->hasFocus())
  	TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->redo();
  if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
    if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->hasFocus())
      TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->redo();
}


void PageCtrl::selectAll()
{ //! select all text in current inputwindow

  if (TabCtrl->count() < 1)
    return; 
  if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->hasFocus())
  	TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->selectAll();
  if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
    if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->hasFocus())
  	  TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->selectAll();
}


void PageCtrl::setNewSplitterVisible()
{ //! set visibility of new created page
	
  if (TabCtrl->count() < 1)
    return;  
    
  TabCtrl->currentWidget()->setVisible(true);  
} 


void PageCtrl::undo()
{ //! undo last typing in current inputwindow

  if (TabCtrl->count() < 1)
    return; 
  if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->hasFocus())
  	TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->undo();
  if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
    if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->hasFocus())
  	  TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->undo();
}




//=============================================================================
// - - - public SIGNALs - - - 
/* EMPTY */




//=============================================================================
// - - - diverse public functions - - - 
QString PageCtrl::activeInputWindow()
{ //! return active inputwindow of current page as short name (Dec, Ins)
	
  if (TabCtrl->count() < 1)
  	return("");
  
  if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->hasFocus())
  	return("Dec");
  if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
    if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->hasFocus())
  	  return("Ins");	
  //if page exists but no textedit has focus, set Instruction section on focus
  this->blockSignals(true);
  if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
  {
    TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->setFocus();
    TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->moveCursor(QTextCursor::Start, QTextCursor::MoveAnchor);
  }
  this->blockSignals(false);
  return("Ins");
}


int PageCtrl::countPages()
{ //! count number of tabs

 return(TabCtrl->count());
}


QString PageCtrl::currentDeclaration()
{ //! return current declaration content

  if (TabCtrl->count() > 0)
    return(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->toPlainText());
  else
    return("");
}


bool PageCtrl::currentDeclarationIsVisible()
{ //! return visibility of current declaration inputwindow
  
  if (TabCtrl->count() < 1)
    return(false);  
    
  return(qobject_cast<QTextEdit*>(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1"))->isVisible());  
}


QString PageCtrl::currentFileEnd()
{ //! return current file end

  if (TabCtrl->count() > 0)
    return(TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_FileEnd);
  else
    return("");
}


int PageCtrl::currentFileExt()
{ //! return curent file extension
  
  if (TabCtrl->count() > 0)
    return((TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_FileExt).toInt());
  else
    return(0);
} 


QString PageCtrl::currentFilePath()
{ //! return curent file path (without file name)
	
  if (TabCtrl->count() > 0)
    return((TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_FilePath).simplified());
  else
    return("");
}


QString PageCtrl::currentFileType()
{ //! return current file type

  if (TabCtrl->count() > 0)
    return((TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_FileType));
  else
    return("");
}


QString PageCtrl::currentFileTypeShort()
{ //! return current file type as short information:
  /*! - nothing set = 'NULL' */
  /*! - set but not detectable = 'error' */
  /*! - PROGRAM or ORGANIZATION_BLOCK = 'PRG' */
  /*! - FUNCTION_BLOCK = 'FB' */
  /*! - FUNCTION = 'FC + currentFncType()' */
  /*! - GLOBAL_VARIABLE_LIST = 'VAR' */
  
  if (TabCtrl->count() < 1)
    return("NULL");
  
  QString str = currentFileType();

  if ((str == "PROGRAM") || (str == "ORGANIZATION_BLOCK"))
    return("PRG");
  if (str == "FUNCTION_BLOCK")
    return("FB");
  if (str == "FUNCTION")
    return("FC" + currentFncType());
  if (str == "GLOBAL_VARIABLE_LIST")
    return("VAR");
  else
  if (str.isEmpty())
    return("NULL");
  else
    return("error");
}
	

QString PageCtrl::currentFncType()
{ //! return current function type

  if (TabCtrl->count() > 0)
    return((TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_FncType).simplified());
  else
    return("");
}


QString PageCtrl::currentInstruction()
{ //! return current instruction content
  
  if ((TabCtrl->count() > 0) && (currentFileType() != "GLOBAL_VARIABLE_LIST"))
    return(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->toPlainText());
  else
    return("");
}
	

QString PageCtrl::currentKMHeader()
{ //return Moeller header of current page
 
  if (TabCtrl->count() < 1)
    return(""); //return if no page is active
     
  return(TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_KMHeader);
}


QString PageCtrl::currentKWDescription()
{ //return KW-Software description of current page
 
  if (TabCtrl->count() < 1)
    return(""); //return if no page is active
    
  return(TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_KWDescription);
}


QString PageCtrl::currentKWProperties()
{ //return KW-Software properties of current page
 
  if (TabCtrl->count() < 1)
    return(""); //return if no page is active
     
  return(TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_KWProperties);
}


QString PageCtrl::currentKWWorkSheetName()
{ //return KW-Software worksheet name of current page
 
  if (TabCtrl->count() < 1)
    return(""); //return if no page is active
     
  return(TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_KWWorkSheetName);
}


QString PageCtrl::currentSiemensDBData()
{ //return Siemens DB Data of current page
 
  if (TabCtrl->count() < 1)
    return(""); //return if no page is active
     
  return(TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_KWWorkSheetName);
}


int PageCtrl::currentTabIndex()
{ //! return current tab/page index

  if (TabCtrl->count() < 1)
    return(-1); //return if no page is active
    
  return(TabCtrl->currentIndex());
}


QString PageCtrl::currentPrgLng()
{ //! return programming language of current page
  
  if (TabCtrl->count() < 1)
    return(""); //return if no page is active

  return(TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_PROGLNG);
}


QString PageCtrl::currentSelection()
{ //! return selected text of current input window
	
  if (TabCtrl->count() < 1)//if < 1 return std. value
  	return("");
  if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->hasFocus())
  	return(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->textCursor().selectedText());
  if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
    if (TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->hasFocus())
  	  return(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->textCursor().selectedText());	
  return("");
}
	

QString	PageCtrl::currentSplitMode()
{ //! return splitter mode of current page

  if (TabCtrl->count() < 1)//if < 1 return std. value
    return("v"); 
  if (qobject_cast<QSplitter *>(TabCtrl->currentWidget())->orientation() == Qt::Horizontal);
    return("v");
  if (qobject_cast<QSplitter *>(TabCtrl->currentWidget())->orientation() == Qt::Vertical);
    return("h");    
}
	
	
int	PageCtrl::currentSplitSize()
{ //! return splitter size of current page

  if (TabCtrl->count() < 1)//if < 1 return std. value
    return(100); 
  
  QList<int> Sizes(qobject_cast<QSplitter *>(TabCtrl->currentWidget())->sizes());  //create list for size values
  return(Sizes.at(0));
}


QString	PageCtrl::currentTabText()
{ //! return current tab/page text

  if (TabCtrl->count() < 1)//if < 1 return std. value
    return(""); 

  return(TabCtrl->tabText(TabCtrl->currentIndex()));
}


QString PageCtrl::cursorPosDec()
{ //! return cursor position of current declaration inputwindow
  int row=0, col=0;
  
  if (TabCtrl->count() > 0)
  {  
    QTextCursor cursor(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->textCursor());
	row = cursor.blockNumber()+1;
	col = cursor.position() - cursor.block().position()+1;  
  }
  return(QString::number(row) + ":" + QString::number(col));
}


QString PageCtrl::cursorPosIns()
{ //! return cursor position of current instruction inputwindow
	
  int row=0, col=0;
  
  if ((TabCtrl->count() > 0) && (currentFileType() != "GLOBAL_VARIABLE_LIST"))
  {  
    QTextCursor cursor(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->textCursor());
	row = cursor.blockNumber()+1;
	col = cursor.position() - cursor.block().position()+1;
  }
  return(QString::number(row) + ":" + QString::number(col));
}


QString PageCtrl::declarationOf(int index)
{ //! return declaration content of tab/page index

  if (TabCtrl->count() < 1)
  return("");
  
  return(TabCtrl->widget(index)->findChild<QTextEdit*>("TE1")->toPlainText());
}


void PageCtrl::delVARTypeItems()
{ //! delete items which are not used in GLOBAL_VAR lists (instruction inputwindow, moeller header)
	
  QTextEdit *TE2 = TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2");
  
  delete    TE2;                         //delete TE2, becc. it will not used
}


void PageCtrl::fillCurrentDeclaration(QString Str)
{ //! clear and fill declaration inputwindow

  qobject_cast<QTextEdit*>(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1"))->setPlainText(Str);
}


void PageCtrl::fillCurrentInstruction(QString Str)
{ //! clear and fill instruction inputwindow

  if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
    qobject_cast<QTextEdit*>(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2"))->setPlainText(Str);
}


void PageCtrl::fillRecentFilesList(int index)
{ //! fill recent files list with item at index
  
  if ((TabCtrl->count() < 1) || (index >= TabCtrl->count()))
    return;
  
  QString str = filePathOf(index) + tabTextOf(index);

  QMutableStringListIterator i(recentFiles);
  while (i.hasNext()) //check if file in list does exist or is already enclosed
    if ((!QFile::exists(i.next())) || (i.peekPrevious() == str))
      i.remove(); 

  if ((!tabTextOf(index).isEmpty()) && (!tabTextOf(index).contains("untitled")))
  { //get current file name and full file path
    recentFiles.prepend(str); //prepend new file
    //check current list length and correct it if needed
    correctRecentFilesListLength();
  }
}


void PageCtrl::correctRecentFilesListLength()
{ //! correct length of recent files list

  if (recentFiles.count() < maxRecentFiles)
    return; //return is current length ok
  
  int cnt = recentFiles.count();
  for (int i=maxRecentFiles; i < cnt; i++)
    recentFiles.removeAt(i);
}


QString PageCtrl::fileEndOf(int index)
{ //! return file end of tab/page index

  if (TabCtrl->count() < 1)
    return("");
    
  return(TabCtrl->widget(index)->findChild<PageData*>("pageData")->pd_FileEnd);
}


int PageCtrl::fileExtOf(int index)
{ //! return file extension of tab/page index
  	
  if (TabCtrl->count() < 1)
    return(0);
    
  return((TabCtrl->widget(index)->findChild<PageData*>("pageData")->pd_FileExt).toInt());
}


QString PageCtrl::fncTypeOf(int index)
{ //! return function type of tab/page index

  if (TabCtrl->count() < 1)
  return("");
  
  return(TabCtrl->widget(index)->findChild<PageData*>("pageData")->pd_FncType);
}


QString PageCtrl::tabTextOf(int index)
{ //! return tab text == file name of tab/page index

  if (TabCtrl->count() < 1)//if < 1 return std. value
    return(""); 

  return(TabCtrl->tabText(TabCtrl->indexOf(TabCtrl->widget(index))));
}


QString PageCtrl::filePathOf(int index)
{ //! return file path of tab/page index
	
  if (TabCtrl->count() < 1)
  return("");
  
  
  return(TabCtrl->widget(index)->findChild<PageData*>("pageData")->pd_FilePath);
}


QString PageCtrl::fileTypeOf(int index)
{ //! return file type of tab/page index
	
  if (TabCtrl->count() < 1)
    return("");

  return(TabCtrl->widget(index)->findChild<PageData*>("pageData")->pd_FileType);
}


bool PageCtrl::find(QString window, QString fStr, QString rStr, QTextDocument::FindFlags options)
{ //! find given fStr or replace with rStr in window by using QTextDocument::FindFlags
  //! \param window must be set to 'Dec' or 'Ins'
  
  if (TabCtrl->count() < 1)
    return(false); 						//return if not tab exists
   
  QTextEdit *TE = 0;					//create a TextEdit pointer
    
  if (window == "Dec")
  { //get TE from Declarations and reset cursor of Instructions
  	TE = TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1"); 
  	if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
  	  TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2")->moveCursor(QTextCursor::Start, QTextCursor::MoveAnchor);
  }
  if (window == "Ins")
  {	//get TE from Instructions and reset cursor of Declarations
  	if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
  	  TE = TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2"); 
  	TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1")->moveCursor(QTextCursor::Start, QTextCursor::MoveAnchor);
  }

  QTextDocument *doc = TE->document();	//get pointer to current document
  QTextCursor c = TE->textCursor();		//get cursor to current text
  
  if (c.atStart() == true)
  	emit cursorAtStartPos();
  
  //find string 
  QTextCursor newCursor = c;
  newCursor = doc->find(fStr, c, options);
  if (newCursor.isNull())
    newCursor = c; 						//if not set old to new cursor
  else
  {
    TE->setTextCursor(newCursor);   	//if yes move text cursor to new cursor  
    if (!rStr.isEmpty())
    {
      newCursor.removeSelectedText();
      newCursor.insertText(rStr);
    }    
  }
  
  //test if the string could be found after accurrent position
  QTextCursor newCursor2 = newCursor;
  newCursor2 = doc->find(fStr, newCursor, options);
  if (newCursor2.isNull())
    return(false); 						//if not return with=0
  else
    return(true);  						//if yes return with=1   
}


void PageCtrl::insertText(QString window, QString Str)
{ //! insert Str into window
  //! \param window must be set to 'Dec' or 'Ins'
  /*! This function calls the function insertTextOf(QString window, QString Str, int index),
   *  by using currentTabIndex() to get the current index.
   *  Inputs on 'Ins' windows will be rejected if file type is GLOBAL_VARIABLE_LIST. 
   */

  if (TabCtrl->count() < 1)
    return; //return if no page is active
  
  insertTextOf(window, Str, currentTabIndex());
}


void PageCtrl::insertTextOf(QString window, QString Str, int index)
{ //! insert Str into window on tab/page index
  //! \param window must be set to 'Dec' or 'Ins'
  /*! Inputs on 'Ins' windows will be rejected if file type is GLOBAL_VARIABLE_LIST. */

  if (TabCtrl->count() < 1)
    return; //return if no page is active
  
  if (window == "Dec")
  { //get acces to TE1, get actual curso pos., insert string
    QTextEdit *TE1 = TabCtrl->widget(index)->findChild<QTextEdit*>("TE1");
    QTextCursor cursor(TE1->textCursor());
    //edit block
    cursor.beginEditBlock();
    cursor.insertText(Str);
    cursor.endEditBlock();
  }
  if ((window == "Ins") && (fileTypeOf(index) != "GLOBAL_VARIABLE_LIST"))
  { //get acces to TE2, get actual curso pos., insert string
    QTextEdit *TE2 = TabCtrl->widget(index)->findChild<QTextEdit*>("TE2");
    QTextCursor cursor(TE2->textCursor());
    //edit block
    cursor.beginEditBlock();
    cursor.insertText(Str);
    cursor.endEditBlock();
  }
}


QString PageCtrl::instructionOf(int index)
{ //! return instruction content of tab/page index

  if (TabCtrl->count() < 1)
    return("");
  
  return(TabCtrl->widget(index)->findChild<QTextEdit*>("TE2")->toPlainText());
}


QString PageCtrl::KMHeaderOf(int index)
{ //! return Moeller header of tab/page index
 
  if (TabCtrl->count() < 1)
    return(""); //return if no page is active
     
  return(TabCtrl->widget(index)->findChild<PageData*>("pageData")->pd_KMHeader);	
}


QString PageCtrl::KWDescriptionOf(int index)
{ //! return KW-Software description section of tab/page index
 
  if (TabCtrl->count() < 1)
    return(""); //return if no page is active
     
  return(TabCtrl->widget(index)->findChild<PageData*>("pageData")->pd_KWDescription);	
}


QString PageCtrl::KWPropertiesOf(int index)
{ //! return KW-Software properties section of tab/page index
 
  if (TabCtrl->count() < 1)
    return(""); //return if no page is active
     
  return(TabCtrl->widget(index)->findChild<PageData*>("pageData")->pd_KWProperties);	
}


QString PageCtrl::KWWorkSheetNameOf(int index)
{ //! return worksheet name of index
	
  if (TabCtrl->count() < 1)
    return("Name: "); //return if no page is active
     
  return(TabCtrl->widget(index)->findChild<PageData*>("pageData")->pd_KWWorkSheetName);
}


QString PageCtrl::SiemensDBDataOf(int index)
{ //! return Siemens DB Data of index
	
  if (TabCtrl->count() < 1)
    return(""); //return if no page is active
     
  return(TabCtrl->widget(index)->findChild<PageData*>("pageData")->pd_SiemensDBData);
}


QString PageCtrl::prgLngOf(int index)
{ //! return programming language of tab/page index
  
  if (TabCtrl->count() < 1)
    return(""); //return if no page is active

  return(TabCtrl->widget(index)->findChild<PageData*>("pageData")->pd_PROGLNG);
}


void PageCtrl::setCurrentDeclarationVisible(bool visible)
{ //! set visibility of current declaration inputwindow 
	  
  if (TabCtrl->count() < 1)
    return;  
    
  qobject_cast<QTextEdit*>(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1"))->setVisible(visible);  
}


void PageCtrl::setCurrentFileEnd(QString Str)
{ //! set file end string of current page

  if (TabCtrl->count() < 1)
    return; //return if no page is active

  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_FileEnd = Str;
}


void PageCtrl::setCurrentFileExt(QString Str)
{ //! set file extension string of current page

  if (TabCtrl->count() < 1)
    return; //return if no page is active

  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_FileExt = Str;
}


void PageCtrl::setCurrentFileListItemText(QString Str)
{ //! insert FileList item named in Str

  if (TabCtrl->count() < 1)
    return; //return if no page is active

  FileList->insertItem(TabCtrl->currentIndex(), Str);
  FileList->takeItem(TabCtrl->currentIndex()+1);
}


void PageCtrl::setCurrentFilePath(QString Str)
{ //! set file path for current page

  if (TabCtrl->count() < 1)
    return; //return if no page is active

  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_FilePath = Str;
}


void PageCtrl::setCurrentFileType(QString Str)
{ //! set file type of current page
  
  if (TabCtrl->count() < 1)
    return; //return if no page is active

  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_FileType = Str;
} 


void PageCtrl::setCurrentFncType(QString Str)
{ //! set function type of current page

  if (TabCtrl->count() < 1)
    return; //return if no page is active

  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_FncType = Str;
}


void PageCtrl::setCurrentFont(QFont font)
{ //! set font of current instruction and declaration inputwindows

  if (TabCtrl->count() < 1)
    return; //return if no page is active
    
  //set Font of TE1
  qobject_cast<QTextEdit*>(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1"))->setFont(font); 
  //set Font of TE2
  if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
    qobject_cast<QTextEdit*>(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2"))->setFont(font); 
}


void PageCtrl::setCurrentKMHeader(QString Str)
{ //! set Moeller header for current tab/page

  if ((TabCtrl->count() < 1) || (currentFileType() == "GLOBAL_VARIABLE_LIST"))
    return; //return if no page is active 

  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_KMHeader = Str;	
}


void PageCtrl::setCurrentKWDescription(QString Str)
{ //! set KW-Software description section for current tab/page
	
  if (TabCtrl->count() < 1)
    return; //return if no page is active

  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_KWDescription = Str; 
}


void PageCtrl::setCurrentKWProperties(QString Str)
{ //! set KW-Software properties section for current tab/page
	
  if (TabCtrl->count() < 1)
    return; //return if no page is active

  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_KWProperties = Str; 
}


void PageCtrl::setCurrentKWWorkSheetName(QString Str)
{ //! set worksheet name for current tab/page
	
  if (TabCtrl->count() < 1)
    return; //return if no page is active

  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_KWWorkSheetName = Str; 
}


void PageCtrl::setCurrentPrgLng(QString Str)
{ //! set programming language for current tab/page
  
  if (TabCtrl->count() < 1)
    return; //return if no page is active

  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_PROGLNG = Str;
}


void PageCtrl::setCurrentSiemensDBData(QString Str)
{ //! set Siemens DB Data for current tab/page
	
  if (TabCtrl->count() < 1)
    return; //return if no page is active

  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_SiemensDBData = Str; 
}


void PageCtrl::setCurrentSplitMode(QString Str)
{ //! set splitter mode of current tab/page

  if (TabCtrl->count() < 1)//if < 1 return std. value
    return; 
   
  if (Str.contains("v"))  //set Splitter orientation
    qobject_cast<QSplitter *>(TabCtrl->currentWidget())->setOrientation(Qt::Horizontal);
  else
    qobject_cast<QSplitter *>(TabCtrl->currentWidget())->setOrientation(Qt::Vertical);

}


void PageCtrl::setCurrentSplitSize(int i)
{ //! set split size for current tab/page
	
  QTextEdit *TE2 = TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2");
  QSplitter *splitter1 = qobject_cast<QSplitter *>(TabCtrl->currentWidget());
  
  QList<int> Sizes;                         //create list for size values
  Sizes << i << 100;                //fill list
  splitter1->setStretchFactor(splitter1->indexOf(TE2), 1); //set stretchfactor for TE2
  splitter1->setSizes(Sizes);               //set size values            	
}


void PageCtrl::setCurrentTypeData(QString filetype, QString functiontype, QString fileendmark)
{ //! set type of file, type of function, end of file string for current tab/page
  
  if (TabCtrl->count() < 1)
    return; //return if no page is active
    
  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_FileType = filetype; 
  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_FncType = functiontype;	
  TabCtrl->currentWidget()->findChild<PageData*>("pageData")->pd_FileEnd = fileendmark;  
}


void PageCtrl::setCurrentTabStops(int width)
{ //! set tab width of current instruction and declaration inputwindows

  if (TabCtrl->count() < 1)
    return; //return if no page is active
    
  //set TabStop width of TE1
  qobject_cast<QTextEdit*>(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE1"))->setTabStopWidth(width); 
  //set TabStop width of TE2
  if ((currentFileType() != "GLOBAL_VARIABLE_LIST"))
    qobject_cast<QTextEdit*>(TabCtrl->currentWidget()->findChild<QTextEdit*>("TE2"))->setTabStopWidth(width); 
}


void PageCtrl::setCurrentTabText(QString Str)
{ //! set tab text of current tab/page

  if (TabCtrl->count() < 1)
    return; //return if no page is active

  TabCtrl->setTabText(TabCtrl->currentIndex(), Str);
}


void PageCtrl::setFileListVisible(bool visible)
{ //! set visibility of FileList

  flg_fileListVisible = visible;
  
  if (TabCtrl->count() < 1)
    return; 
  
  FileList->setVisible(visible);		//set file list visible
  FileList->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred);
  //FileList->setGeometry(0, 0, FileList->minimumWidth(), TabCtrl->height());
}


void PageCtrl::setKWPropertiesOf(int index, QString Str)
{ //! set KW-Software properties section of index
	
  if (TabCtrl->count() < 1)
    return; //return if no page is active

  TabCtrl->widget(index)->findChild<PageData*>("pageData")->pd_KWProperties = Str; 
}


void PageCtrl::setPrgLngOf(int index, QString Str)
{ //! set programming language of index
  
  if (TabCtrl->count() < 1)
    return; //return if no page is active

  TabCtrl->widget(index)->findChild<PageData*>("pageData")->pd_PROGLNG = Str;
}


void PageCtrl::setSplitModeOf(int index, QString Str)
{ //! set split mode of index

  if (TabCtrl->count() < 1)//if < 1 return std. value
    return; 
   
  if (Str.contains("v"))  //set Splitter orientation
    qobject_cast<QSplitter *>(TabCtrl->widget(index))->setOrientation(Qt::Horizontal);
  else
    qobject_cast<QSplitter *>(TabCtrl->widget(index))->setOrientation(Qt::Vertical);
}


void PageCtrl::setTabCtrlVisible(bool visible)
{ //! set visibility of TabCtrl

  TabCtrl->setVisible(visible);		//set tab ctrl. visible
  TabCtrl->setGeometry(FileList->width(), 0, TabCtrl->maximumWidth(), TabCtrl->height());
}


void PageCtrl::setTabPosition(int pos)
{ //! set tab orientation by selected postion
  //! \param pos 1=north, 2=south, 3=east, 4=west, default is north

  switch ( pos )
  {
    case 1  : { //north position
  	            TabCtrl->setTabPosition(QTabWidget::North);//set Tab pos. to north
                break;                                     //leave case contruct
              } //end case 1
    case 2  : { //south position
  	            TabCtrl->setTabPosition(QTabWidget::South);//set Tab pos. to north
                break;                                     //leave case contruct
              } //end case 2
    case 3  : { //east position
  	            TabCtrl->setTabPosition(QTabWidget::East);//set Tab pos. to north
                break;                                    //leave case contruct
              } //end case 3
    case 4  : { //west position
  	            TabCtrl->setTabPosition(QTabWidget::West);//set Tab pos. to north
                break;                                    //leave case contruct
              } //end case 4
   default :  { //empty
                TabCtrl->setTabPosition(QTabWidget::North);//set Tab pos. to north
                break;                                     //leave case contruct
              } //end default
  }//end switch
}




//=============================================================================
// - - - private SLOTs - - -
void PageCtrl::setFileListRow(int i)
{ //! slot for set FileList row by i
	
  FileList->setCurrentRow(i);	
}


void PageCtrl::cursorPositionChanged()
{ //! slot curosr postion changed, does emit cursor modified signal
	
  emit cursorModified();
}




//=============================================================================
// - - - diverse private functions - - - 
QString PageCtrl::strippedFileName(const QString &fullFileName)
{ //! strip filename from fullFileName (path + filename)
  
  return(QFileInfo(fullFileName).fileName().simplified());
}


void PageCtrl::retranslateUi()
{ //! retranslate Ui
	
  delPageButton->setToolTip(tr("Close page"));
}
